package cz.muni.fi.pa165.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configurers.oauth2.server.resource.OAuth2ResourceServerConfigurer;
import org.springframework.security.web.SecurityFilterChain;

@Configuration
public class SecurityConfiguration {

    @Bean
    SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http.authorizeHttpRequests(x -> x
                        .requestMatchers(HttpMethod.GET, "/api/v1/bands/v3/api-docs").permitAll()
                        .requestMatchers(HttpMethod.GET, "/swagger-ui/**").permitAll()
                        .requestMatchers(HttpMethod.GET, "/v3/api-docs/**").permitAll()
                        .requestMatchers(HttpMethod.GET, "/v3/api-docs/**").permitAll()
                        .requestMatchers(HttpMethod.GET, "/api/v1/bands/v3/api-docs/swagger-config").permitAll()
                        .requestMatchers(HttpMethod.GET, "/api/v1/bands/**").hasAuthority("SCOPE_test_read")
                        .requestMatchers(HttpMethod.POST, "/api/v1/bands/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PUT, "/api/v1/bands/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.DELETE, "/api/v1/bands/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PATCH, "/api/v1/bands/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.GET, "/api/v1/songs/v3/api-docs/swagger-config").permitAll()
                        .requestMatchers(HttpMethod.GET, "/api/v1/songs/**").hasAuthority("SCOPE_test_read")
                        .requestMatchers(HttpMethod.POST, "/api/v1/songs/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PUT, "/api/v1/songs/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.DELETE, "/api/v1/songs/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PATCH, "/api/v1/songs/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.GET, "/api/v1/albums/v3/api-docs/swagger-config").permitAll()
                        .requestMatchers(HttpMethod.GET, "/api/v1/albums/**").hasAuthority("SCOPE_test_read")
                        .requestMatchers(HttpMethod.POST, "/api/v1/albums/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PUT, "/api/v1/albums/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.DELETE, "/api/v1/albums/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PATCH, "/api/v1/albums/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.GET, "/api/v1/tours/v3/api-docs/swagger-config").permitAll()
                        .requestMatchers(HttpMethod.GET, "/api/v1/tours/**").hasAuthority("SCOPE_test_read")
                        .requestMatchers(HttpMethod.POST, "/api/v1/tours/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PUT, "/api/v1/tours/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.DELETE, "/api/v1/tours/**").hasAuthority("SCOPE_test_write")
                        .requestMatchers(HttpMethod.PATCH, "/api/v1/tours/**").hasAuthority("SCOPE_test_write")
                        .anyRequest().authenticated()
                )
                .oauth2ResourceServer(OAuth2ResourceServerConfigurer::opaqueToken);
        return http.build();
    }
}
