package cz.muni.fi.pa165.controller.album;

import cz.muni.fi.pa165.model.album.AlbumCreateDTO;
import cz.muni.fi.pa165.model.album.AlbumDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RequestMapping("/api/v1/albums")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Album Management Service API", version = "1.0",
        description = "API used for performing CRUD operations on Album"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Album Management Service", description = "APIs for managing albums")
public interface AlbumController {

    @PostMapping
    @Operation(summary = "Create new album")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<AlbumDTO> createAlbum(@RequestBody @Valid AlbumCreateDTO albumDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get album by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<AlbumDTO> getAlbumById(@PathVariable("id") long id);

    @PutMapping("/{id}")
    @Operation(summary = "Update existing album by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<AlbumDTO> updateAlbumById(@PathVariable("id") long id, @RequestBody @Valid AlbumDTO albumDTO);

    @DeleteMapping("/{id}")
    @Operation(summary = "Remove existing album by ID")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<Void> removeAlbumById(@PathVariable("id") long id);

}
