package cz.muni.fi.pa165.controller.band;

import cz.muni.fi.pa165.model.band.BandDTO;
import cz.muni.fi.pa165.model.band.BandCreateDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.oauth2.server.resource.authentication.BearerTokenAuthentication;
import org.springframework.web.bind.annotation.*;

@RequestMapping("/api/v1/bands")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Band Management Service API", version = "1.0",
        description = "API used for performing CRUD operations on Band"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Band Management Service", description = "APIs for managing bands")
public interface BandController {

    @PostMapping
    @Operation(summary = "Create new band")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<BandDTO> createBand(@RequestBody @Valid BandCreateDTO bandDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get band by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<BandDTO> getBandById(@PathVariable("id") long id);

    @PutMapping("/{id}")
    @Operation(summary = "Update existing band by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<BandDTO> updateBandById(@PathVariable("id") long id, @RequestBody @Valid BandDTO bandDTO);

    @DeleteMapping("/{id}")
    @Operation(summary = "Remove existing band by ID")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<Void> removeBandById(@PathVariable("id") long id);

    @PostMapping("/{id}/addTeamMember/{teamMemberId}")
    @Operation(summary = "Add team member to band")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<Void> addTeamMemberToBand(@PathVariable("id") long id, @PathVariable("teamMemberId") long teamMemberId);

}
