package cz.muni.fi.pa165.controller.band;

import cz.muni.fi.pa165.model.band.BandDTO;
import cz.muni.fi.pa165.model.band.BandCreateDTO;
import cz.muni.fi.pa165.service.band.BandService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.oauth2.server.resource.authentication.BearerTokenAuthentication;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class BandControllerImpl implements BandController {

    private final BandService bandService;

    @Autowired
    public BandControllerImpl(BandService bandService) {
        this.bandService = bandService;
    }

    @Override
    public ResponseEntity<BandDTO> createBand(BandCreateDTO bandDTO) {
        return ResponseEntity.status(HttpStatus.CREATED).body(bandService.createBand(bandDTO));
    }

    @Override
    public ResponseEntity<BandDTO> getBandById(long id) {
        return ResponseEntity.ok(bandService.getBandById(id));
    }

    @Override
    public ResponseEntity<BandDTO> updateBandById(long id, BandDTO bandDTO) {
        return ResponseEntity.ok(bandService.updateBandById(id, bandDTO));
    }

    @Override
    public ResponseEntity<Void> removeBandById(long id) {
        bandService.removeBandById(id);
        return ResponseEntity.noContent().build();
    }

    @Override
    public ResponseEntity<Void> addTeamMemberToBand(long id, long teamMemberId) {
        bandService.addTeamMemberToBand(id, teamMemberId);
        return ResponseEntity.noContent().build();
    }

}
