package cz.muni.fi.pa165.controller.song;

import cz.muni.fi.pa165.model.song.SongDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RequestMapping("/api/v1/songs")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Song Management Service API", version = "1.0",
                description = "API used for performing CRUD operations on Song"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Song Management Service", description = "APIs for managing songs")
public interface SongController {

    @PostMapping
    @Operation(summary = "Create new song")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<SongDTO> createSong(@RequestBody @Valid SongDTO songDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get song by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<SongDTO> getSongById(@PathVariable("id") long id);

    @PutMapping("/{id}")
    @Operation(summary = "Update existing song by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<SongDTO> updateSongById(@PathVariable("id") long id, @RequestBody @Valid SongDTO songDTO);

    @DeleteMapping("/{id}")
    @Operation(summary = "Remove existing song by ID")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<Void> removeSongById(@PathVariable("id") long id);

}
