package cz.muni.fi.pa165.controller.tour;

import cz.muni.fi.pa165.data.entity.TourStop;
import cz.muni.fi.pa165.model.tour.TourCreateDTO;
import cz.muni.fi.pa165.model.tour.TourDTO;
import cz.muni.fi.pa165.model.tour.TourStopDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RequestMapping("/api/v1/tours")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Tour Management Service API", version = "1.0",
        description = "API used for performing CRUD operations on Tour"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Tour Management Service", description = "APIs for managing tours")
public interface TourController {

    @PostMapping
    @Operation(summary = "Create new tour")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<TourDTO> createTour(@RequestBody @Valid TourCreateDTO tourCreateDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get tour by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<TourDTO> getTourById(@PathVariable("id") long id);

    @PutMapping("/{id}")
    @Operation(summary = "Update existing tour by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<TourDTO> updateTourById(@PathVariable("id") long id, @RequestBody @Valid TourDTO tourDTO);

    @DeleteMapping("/{id}")
    @Operation(summary = "Remove existing tour by ID")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<Void> removeTourById(@PathVariable("id") long id);

    @PostMapping("/{id}/stops")
    @Operation(summary = "Create new tour stop")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<TourStopDTO> createTourStop(@PathVariable("id") long id, @RequestBody @Valid TourStopDTO tourStopDTO);

    @DeleteMapping("/{id}/stops/{stopId}")
    @Operation(summary = "Delete tour stop")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<Void> removeTourStop(@PathVariable("id") long id, @PathVariable("stopId") long tourStopId);

    @GetMapping("/{id}/stops")
    @Operation(summary = "Get all tour stops for specific tour")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<List<TourStopDTO>> getAllTourStops(@PathVariable("id") long id);
}