package cz.muni.fi.pa165.controller.tour;

import cz.muni.fi.pa165.model.tour.TourCreateDTO;
import cz.muni.fi.pa165.model.tour.TourDTO;
import cz.muni.fi.pa165.model.tour.TourStopDTO;
import cz.muni.fi.pa165.service.tour.TourService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
public class TourControllerImpl implements TourController {

    private final TourService tourService;

    @Autowired
    public TourControllerImpl(TourService tourService) {
        this.tourService = tourService;
    }

    @Override
    public ResponseEntity<TourDTO> createTour(TourCreateDTO tourCreateDTO) {
        return ResponseEntity.status(HttpStatus.CREATED).body(tourService.createTour(tourCreateDTO));
    }

    @Override
    public ResponseEntity<TourDTO> getTourById(long id) {
        return ResponseEntity.ok(tourService.getTourById(id));
    }

    @Override
    public ResponseEntity<TourDTO> updateTourById(long id, TourDTO tourDTO) {
        return ResponseEntity.ok(tourService.updateTourById(id, tourDTO));
    }

    @Override
    public ResponseEntity<Void> removeTourById(long id) {
        tourService.removeTourById(id);
        return ResponseEntity.noContent().build();
    }

    @Override
    public ResponseEntity<TourStopDTO> createTourStop(long id, TourStopDTO tourStopDTO) {
        return ResponseEntity.status(HttpStatus.CREATED).body(tourService.createTourStop(id, tourStopDTO));
    }

    @Override
    public ResponseEntity<Void> removeTourStop(long id, long tourStopId) {
        tourService.deleteTourStop(id, tourStopId);
        return ResponseEntity.noContent().build();
    }

    @Override
    public ResponseEntity<List<TourStopDTO>> getAllTourStops(long id) {
        return ResponseEntity.ok(tourService.getAllTourStops(id));
    }

}
