package cz.muni.fi.pa165.mapper;

import cz.muni.fi.pa165.data.entity.Album;
import cz.muni.fi.pa165.data.entity.Band;
import cz.muni.fi.pa165.data.entity.Song;
import cz.muni.fi.pa165.data.repository.album.AlbumRepository;
import cz.muni.fi.pa165.model.album.AlbumCreateDTO;
import cz.muni.fi.pa165.model.album.AlbumDTO;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.NullValueMappingStrategy;
import org.mapstruct.factory.Mappers;

import java.util.List;
import java.util.stream.Collectors;

@Mapper(componentModel = "spring", nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT)
public interface AlbumMapper {

    AlbumMapper INSTANCE = Mappers.getMapper(AlbumMapper.class);

    @Mapping(target = "songIds", source = "songs", qualifiedByName = "mapSongsToLongList")
    @Mapping(target = "bandId", source = "band", qualifiedByName = "mapBandToLong")
    AlbumDTO albumToAlbumDTO(Album album);

    Album albumDTOToAlbum(AlbumDTO albumDTO);

    Album albumCreateDTOToAlbum(AlbumCreateDTO albumCreateDTO);

    @Named("mapSongsToLongList")
    default List<Long> mapSongsToLongList(List<? extends Song> entities) {
        if (entities == null) {
            return null;
        }
        return entities.stream().map(Song::getId).collect(Collectors.toList());
    }

    @Named("mapBandToLong")
    default Long mapBandToLong(Band entity) {
        if (entity == null) {
            return null;
        }
        return entity.getId();
    }
    
}
