package cz.muni.fi.pa165.mapper;

import cz.muni.fi.pa165.data.entity.Album;
import cz.muni.fi.pa165.data.entity.Band;
import cz.muni.fi.pa165.data.entity.Tour;
import cz.muni.fi.pa165.model.band.BandDTO;
import cz.muni.fi.pa165.model.band.BandCreateDTO;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.NullValueMappingStrategy;
import org.mapstruct.factory.Mappers;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Mapper(componentModel = "spring", nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT)
public interface BandMapper {

    BandMapper INSTANCE = Mappers.getMapper(BandMapper.class);

    @Mapping(target = "tourIds", source = "tours", qualifiedByName = "mapToursToLongList")
    @Mapping(target = "albumIds", source = "albums", qualifiedByName = "mapAlbumsToLongList")
    BandDTO bandToBandDTO(Band band);

    Band bandDTOToBand(BandDTO bandDTO);


    Band bandCreateDTOToBand(BandCreateDTO bandCreateDTO);

    @Named("mapAlbumsToLongList")
    default List<Long> mapAlbumsToLongList(List<? extends Album> entities) {
        if (entities == null) {
            return null;
        }
        if (entities.size() == 0) {
            return Collections.emptyList();
        }
        return entities.stream().map(Album::getId).collect(Collectors.toList());
    }

    @Named("mapToursToLongList")
    default List<Long> mapToursToLongList(List<? extends Tour> entities) {
        if (entities == null) {
            return null;
        }
        if (entities.size() == 0) {
            return Collections.emptyList();
        }
        return entities.stream().map(Tour::getId).collect(Collectors.toList());
    }
}
