package cz.muni.fi.pa165.service.album;

import cz.muni.fi.pa165.data.entity.Band;
import cz.muni.fi.pa165.data.entity.Song;
import cz.muni.fi.pa165.data.repository.band.BandRepository;
import cz.muni.fi.pa165.data.repository.song.SongRepository;
import cz.muni.fi.pa165.exception.model.BandNotFoundException;
import cz.muni.fi.pa165.exception.model.SongNotFoundException;
import cz.muni.fi.pa165.mapper.AlbumMapper;
import cz.muni.fi.pa165.model.album.AlbumCreateDTO;
import cz.muni.fi.pa165.model.album.AlbumDTO;
import cz.muni.fi.pa165.data.entity.Album;
import cz.muni.fi.pa165.exception.model.AlbumNotFoundException;
import cz.muni.fi.pa165.data.repository.album.AlbumRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class AlbumServiceImpl implements AlbumService {

    private final AlbumRepository albumRepository;

    private final SongRepository songRepository;

    private final BandRepository bandRepository;

    private final AlbumMapper albumMapper;

    @Autowired
    public AlbumServiceImpl(AlbumRepository albumRepository, SongRepository songRepository, BandRepository bandRepository, AlbumMapper albumMapper) {
        this.albumRepository = albumRepository;
        this.songRepository = songRepository;
        this.bandRepository = bandRepository;
        this.albumMapper = albumMapper;
    }

    @Override
    public AlbumDTO createAlbum(AlbumCreateDTO albumCreateDTO) {
        Album mapped = albumMapper.albumCreateDTOToAlbum(albumCreateDTO);

        Band found = bandRepository.findById(albumCreateDTO.getBandId()).orElseThrow(
                () -> new BandNotFoundException(String.format("Band with id %s was not found", albumCreateDTO.getBandId()))
        );

        mapped.setBand(found);

        Album saved = albumRepository.save(mapped);
        return albumMapper.albumToAlbumDTO(saved);
    }

    @Override
    public AlbumDTO getAlbumById(long id) {
        Album found = albumRepository.findById(id).orElseThrow(
                () -> new AlbumNotFoundException(String.format("Album with id %s was not found", id))
        );
        return albumMapper.albumToAlbumDTO(found);
    }

    @Override
    public AlbumDTO updateAlbumById(long id, AlbumDTO albumDTO) {
        albumDTO.setId(id);

        Album mappedAlbum = albumMapper.albumDTOToAlbum(albumDTO);

        // Find corresponding songs and add them
        List<Song> foundSongs = new ArrayList<>();
        albumDTO.getSongIds().forEach(
                songId -> foundSongs.add(songRepository.findById(songId).orElseThrow(
                        () -> new SongNotFoundException(String.format("Song with id %s was not found", songId))
                ))
        );
        mappedAlbum.setSongs(foundSongs);

        Album saved = albumRepository.save(mappedAlbum);
        
        return albumMapper.albumToAlbumDTO(saved);
    }

    @Override
    public void removeAlbumById(long id) {
        Album found = albumRepository.findById(id).orElseThrow(
                () -> new AlbumNotFoundException(String.format("Album with id %s was not found", id))
        );
        albumRepository.delete(found);
    }

}
