package cz.muni.fi.pa165.service.song;

import cz.muni.fi.pa165.data.entity.Album;
import cz.muni.fi.pa165.data.entity.Song;
import cz.muni.fi.pa165.data.repository.album.AlbumRepository;
import cz.muni.fi.pa165.exception.model.AlbumNotFoundException;
import cz.muni.fi.pa165.exception.model.SongNotFoundException;
import cz.muni.fi.pa165.mapper.SongMapper;
import cz.muni.fi.pa165.model.song.SongDTO;
import cz.muni.fi.pa165.data.repository.song.SongRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SongServiceImpl implements SongService {

    private final SongRepository songRepository;

    private final AlbumRepository albumRepository;

    private final SongMapper songMapper;

    @Autowired
    public SongServiceImpl(SongRepository songRepository, AlbumRepository albumRepository, SongMapper songMapper) {
        this.songRepository = songRepository;
        this.albumRepository = albumRepository;
        this.songMapper = songMapper;
    }

    private Album findAlbumById(long id) {
        return albumRepository.findById(id).orElseThrow(
                () -> new AlbumNotFoundException(String.format("Album with id %s was not found", id))
        );
    }

    @Override
    public SongDTO createSong(SongDTO songDTO) {
        Song mappedSong = songMapper.songDTOToSong(songDTO);

        // Find corresponding album and change it
        Album foundAlbum = findAlbumById(songDTO.getAlbumId());
        mappedSong.setAlbum(foundAlbum);

        songRepository.save(mappedSong);

        return songMapper.songToSongDTO(mappedSong);
    }

    @Override
    public SongDTO getSongById(long id) {
        Song found = songRepository.findById(id).orElseThrow(
                () -> new SongNotFoundException(String.format("Song with id %s was not found", id))
        );
        return songMapper.songToSongDTO(found);
    }

    @Override
    public SongDTO updateSongById(long id, SongDTO songDTO) {
        songDTO.setId(id);

        Song mappedSong = songMapper.songDTOToSong(songDTO);

        // Find corresponding album and change it
        Album foundAlbum = findAlbumById(songDTO.getAlbumId());
        mappedSong.setAlbum(foundAlbum);

        Song updated = songRepository.save(mappedSong);

        return songMapper.songToSongDTO(updated);
    }

    @Override
    public void removeSongById(long id) {
        Song found = songRepository.findById(id).orElseThrow(
                () -> new SongNotFoundException(String.format("Song with id %s was not found", id))
        );
        songRepository.delete(found);
    }

}
