package cz.muni.fi.pa165.service.tour;

import cz.muni.fi.pa165.data.entity.Band;
import cz.muni.fi.pa165.data.entity.Tour;
import cz.muni.fi.pa165.data.entity.TourStop;
import cz.muni.fi.pa165.data.repository.band.BandRepository;
import cz.muni.fi.pa165.data.repository.tourstop.TourStopRepository;
import cz.muni.fi.pa165.exception.model.BandNotFoundException;
import cz.muni.fi.pa165.exception.model.TourNotFoundException;
import cz.muni.fi.pa165.exception.model.TourStopNotFoundException;
import cz.muni.fi.pa165.mapper.TourMapper;
import cz.muni.fi.pa165.mapper.TourStopMapper;
import cz.muni.fi.pa165.model.tour.TourCreateDTO;
import cz.muni.fi.pa165.model.tour.TourDTO;
import cz.muni.fi.pa165.data.repository.tour.TourRepository;
import cz.muni.fi.pa165.model.tour.TourStopDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class TourServiceImpl implements TourService {

    private final TourRepository tourRepository;

    private final TourStopRepository tourStopRepository;

    private final BandRepository bandRepository;

    private final TourMapper tourMapper;

    private final TourStopMapper tourStopMapper;

    @Autowired
    public TourServiceImpl(TourRepository tourRepository, TourStopRepository tourStopRepository, BandRepository bandRepository, TourMapper tourMapper, TourStopMapper tourStopMapper) {
        this.tourRepository = tourRepository;
        this.tourStopRepository = tourStopRepository;
        this.bandRepository = bandRepository;
        this.tourMapper = tourMapper;
        this.tourStopMapper = tourStopMapper;
    }

    @Override
    public TourDTO createTour(TourCreateDTO tourCreateDTO) {
        Tour mappedTour = tourMapper.tourCreateDTOToTour(tourCreateDTO);

        Band found = bandRepository.findById(tourCreateDTO.getBandId()).orElseThrow(
                () -> new BandNotFoundException(String.format("Band with id %s was not found", tourCreateDTO.getBandId()))
        );
        mappedTour.setBand(found);

        Tour saved = tourRepository.save(mappedTour);

        return tourMapper.tourToTourDTO(saved);
    }

    @Override
    public TourDTO getTourById(long id) {
        Tour found = tourRepository.findById(id).orElseThrow(
                () -> new TourNotFoundException(String.format("Tour with id %s was not found", id))
        );
        return tourMapper.tourToTourDTO(found);
    }

    @Override
    public TourDTO updateTourById(long id, TourDTO tourDTO) {
        Tour updated = tourRepository.save(tourMapper.tourDTOToTour(tourDTO));
        return tourMapper.tourToTourDTO(updated);
    }

    @Override
    public void removeTourById(long id) {
        Tour found = tourRepository.findById(id).orElseThrow(
                () -> new TourNotFoundException(String.format("Tour with id %s was not found", id))
        );
        tourRepository.delete(found);
    }

    @Override
    public TourStopDTO createTourStop(long id, TourStopDTO tourStopDTO) {
        Tour found = tourRepository.findById(id).orElseThrow(
                () -> new TourNotFoundException(String.format("Tour with id %s was not found", id))
        );

        TourStop mappedTourStop = tourStopMapper.tourStopDTOToTourStop(tourStopDTO);
        mappedTourStop.setTour(found);

        TourStop saved = tourStopRepository.save(mappedTourStop);

        return tourStopMapper.tourStopToTourStopDTO(saved);
    }

    @Override
    public TourDTO deleteTourStop(long id, long tourStopId) {
        TourStop found = tourStopRepository.findById(tourStopId).orElseThrow(
                () -> new TourStopNotFoundException(String.format("Tour stop with id %s not found", id))
        );

        tourStopRepository.delete(found);

        return getTourById(id);
    }

    @Override
    public List<TourStopDTO> getAllTourStops(long id) {
        Tour found = tourRepository.findById(id).orElseThrow(
                () -> new TourStopNotFoundException(String.format("Tour with id %s not found", id))
        );

        return found.getTourStops().stream().map(tourStopMapper::tourStopToTourStopDTO).toList();
    }

}
