package cz.muni.fi.pa165.service.band;

import cz.muni.fi.pa165.data.entity.Album;
import cz.muni.fi.pa165.data.entity.Band;
import cz.muni.fi.pa165.data.entity.Tour;
import cz.muni.fi.pa165.data.repository.album.AlbumRepository;
import cz.muni.fi.pa165.data.repository.band.BandRepository;
import cz.muni.fi.pa165.data.repository.tour.TourRepository;
import cz.muni.fi.pa165.exception.model.AlbumNotFoundException;
import cz.muni.fi.pa165.exception.model.BandNotFoundException;
import cz.muni.fi.pa165.exception.model.TourNotFoundException;
import cz.muni.fi.pa165.mapper.BandMapper;
import cz.muni.fi.pa165.model.band.BandCreateDTO;
import cz.muni.fi.pa165.model.band.BandDTO;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.*;

public class BandServiceImplTest {

    @Mock
    private BandRepository bandRepository;

    @Mock
    private AlbumRepository albumRepository;

    @Mock
    private TourRepository tourRepository;

    @Mock
    private BandMapper bandMapper;

    @Mock
    private KafkaTemplate kafkaTemplate;

    @Mock
    private RestTemplate restTemplate;

    private BandServiceImpl bandService;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.openMocks(this);
        bandService = new BandServiceImpl(bandRepository, albumRepository, tourRepository, bandMapper, kafkaTemplate, restTemplate);
    }

    /*@Test
    public void createBandTest() {
        BandCreateDTO bandCreateDTO = new BandCreateDTO();
        bandCreateDTO.setName("Test Band");

        Band createdBand = new Band();
        createdBand.setId(1L);
        createdBand.setName(bandCreateDTO.getName());

        BandDTO createdDTO = new BandDTO();
        createdDTO.setId(createdBand.getId());
        createdDTO.setName(createdBand.getName());

        when(bandRepository.save(bandMapper.bandCreateDTOToBand(bandCreateDTO))).thenReturn(createdBand);
        when(bandMapper.bandToBandDTO(createdBand)).thenReturn(createdDTO);

        BandDTO result = bandService.createBand(bandCreateDTO);

        assertEquals(createdDTO, result);
    }*/

    @Test
    public void getBandByIdTest() {
        long id = 1L;

        Band foundBand = new Band();
        foundBand.setId(id);
        foundBand.setName("Test Band");

        BandDTO foundDTO = new BandDTO();
        foundDTO.setId(foundBand.getId());
        foundDTO.setName(foundBand.getName());

        when(bandRepository.findById(id)).thenReturn(Optional.of(foundBand));
        when(bandMapper.bandToBandDTO(foundBand)).thenReturn(foundDTO);

        BandDTO result = bandService.getBandById(id);

        assertEquals(foundDTO, result);
    }

    @Test
    public void getBandByIdNotFoundExceptionTest() {
        long id = 1L;

        when(bandRepository.findById(id)).thenReturn(Optional.empty());

        assertThrows(BandNotFoundException.class, () -> bandService.getBandById(id));
    }

    @Test
    public void testUpdateBandById() {
        BandDTO bandDTO = new BandDTO();
        bandDTO.setId(1L);
        bandDTO.setName("Test Band");
        bandDTO.setAlbumIds(Arrays.asList(1L, 2L));
        bandDTO.setTourIds(Arrays.asList(3L, 4L));

        Album album1 = new Album();
        album1.setId(1L);
        album1.setName("Test Album 1");

        Album album2 = new Album();
        album2.setId(2L);
        album2.setName("Test Album 2");

        Tour tour3 = new Tour();
        tour3.setId(3L);
        tour3.setName("Test Tour 3");

        Tour tour4 = new Tour();
        tour4.setId(4L);
        tour4.setName("Test Tour 4");

        when(albumRepository.findById(1L)).thenReturn(Optional.of(album1));
        when(albumRepository.findById(2L)).thenReturn(Optional.of(album2));
        when(tourRepository.findById(3L)).thenReturn(Optional.of(tour3));
        when(tourRepository.findById(4L)).thenReturn(Optional.of(tour4));

        Band mappedBand = new Band();
        mappedBand.setId(1L);
        mappedBand.setName("Test Band");

        when(bandMapper.bandDTOToBand(bandDTO)).thenReturn(mappedBand);

        Band updatedBand = new Band();
        updatedBand.setId(1L);
        updatedBand.setName("Test Band");
        updatedBand.setAlbums(Arrays.asList(album1, album2));
        updatedBand.setTours(Arrays.asList(tour3, tour4));

        when(bandRepository.save(mappedBand)).thenReturn(updatedBand);

        when(bandMapper.bandToBandDTO(updatedBand)).thenReturn(bandDTO);

        BandDTO result = bandService.updateBandById(1L, bandDTO);

        Mockito.verify(albumRepository, times(2)).findById(anyLong());
        Mockito.verify(tourRepository, times(2)).findById(anyLong());
        Mockito.verify(bandRepository, times(1)).save(mappedBand);

        assertEquals(1L, result.getId());
        assertEquals("Test Band", result.getName());
        assertEquals(Arrays.asList(1L, 2L), result.getAlbumIds());
        assertEquals(Arrays.asList(3L, 4L), result.getTourIds());
    }

    @Test
    public void testRemoveBandById() {
        long id = 1L;

        Mockito.doNothing().when(bandRepository).deleteById(id);

        bandService.removeBandById(id);

        Mockito.verify(bandRepository, times(1)).deleteById(id);
    }

    @Test
    void updateBandByIdAlbumNotFound() {
        BandDTO bandDTO = new BandDTO();
        bandDTO.setId(1L);
        bandDTO.setAlbumIds(List.of(1L, 2L));
        bandDTO.setTourIds(List.of(1L, 2L));

        when(albumRepository.findById(anyLong())).thenReturn(Optional.empty());

        BandServiceImpl bandService = new BandServiceImpl(bandRepository, albumRepository, tourRepository, bandMapper, kafkaTemplate, restTemplate);

        assertThrows(AlbumNotFoundException.class, () ->
                bandService.updateBandById(1L, bandDTO)
        );
    }

    @Test
    void updateBandByIdTourNotFound() {
        BandDTO bandDTO = new BandDTO();
        bandDTO.setId(1L);
        bandDTO.setAlbumIds(List.of());
        bandDTO.setTourIds(List.of(1L, 2L));

        when(tourRepository.findById(anyLong())).thenReturn(Optional.empty());
        when(bandMapper.bandDTOToBand(bandDTO)).thenReturn(new Band());

        BandServiceImpl bandService = new BandServiceImpl(bandRepository, albumRepository, tourRepository, bandMapper, kafkaTemplate, restTemplate);

        assertThrows(TourNotFoundException.class, () ->
                bandService.updateBandById(1L, bandDTO)
        );
    }

    @Test
    void addTeamMemberToBand() {
        long id = 1L;
        long teamMemberId = 2L;

        BandDTO bandDTO = new BandDTO();
        bandDTO.setId(id);
        bandDTO.setBandMemberIds(new ArrayList<>());
        bandDTO.setAlbumIds(new ArrayList<>());
        bandDTO.setTourIds(new ArrayList<>());

        Band band = new Band();

        when(bandRepository.findById(id)).thenReturn(Optional.of(band));
        when(bandMapper.bandToBandDTO(band)).thenReturn(bandDTO);
        when(bandMapper.bandDTOToBand(any(BandDTO.class))).thenReturn(band);

        bandService.addTeamMemberToBand(id, teamMemberId);

        // We only want to test that teamMemberId was added to BandDTO, other
        // methods are already well tested
        assertTrue(bandDTO.getBandMemberIds().contains(teamMemberId));

        verify(bandRepository, times(1)).findById(id);
        verify(bandRepository, times(1)).save(any(Band.class));
    }

}