package cz.muni.fi.pa165.service.song;

import cz.muni.fi.pa165.data.entity.Album;
import cz.muni.fi.pa165.data.entity.Song;
import cz.muni.fi.pa165.data.repository.album.AlbumRepository;
import cz.muni.fi.pa165.data.repository.song.SongRepository;
import cz.muni.fi.pa165.mapper.SongMapper;
import cz.muni.fi.pa165.model.song.SongDTO;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;

class SongServiceImplTest {

    @Mock
    private SongRepository songRepository;

    @Mock
    private AlbumRepository albumRepository;

    @Mock
    private SongMapper songMapper;

    private SongServiceImpl songService;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        songService = new SongServiceImpl(songRepository, albumRepository, songMapper);
    }

    @Test
    @DisplayName("Create Song Test")
    void createSongTest() {
        SongDTO songDTO = new SongDTO();
        songDTO.setAlbumId(1L);

        Song song = new Song();
        song.setAlbum(new Album());

        when(albumRepository.findById(1L)).thenReturn(Optional.of(new Album()));
        when(songMapper.songDTOToSong(songDTO)).thenReturn(song);
        when(songRepository.save(song)).thenReturn(song);
        when(songMapper.songToSongDTO(song)).thenReturn(songDTO);

        SongDTO createdSong = songService.createSong(songDTO);

        assertNotNull(createdSong);
        assertEquals(1L, createdSong.getAlbumId());
    }

    @Test
    @DisplayName("Get Song By Id Test")
    void getSongByIdTest() {
        Song song = new Song();
        song.setId(1L);

        SongDTO songDTO = new SongDTO();
        songDTO.setId(1L);

        when(songRepository.findById(1L)).thenReturn(Optional.of(song));
        when(songMapper.songToSongDTO(song)).thenReturn(songDTO);

        SongDTO retrievedSong = songService.getSongById(1L);

        assertNotNull(retrievedSong);
        assertEquals(1L, retrievedSong.getId());
    }

    @Test
    @DisplayName("Update Song By Id Test")
    void updateSongByIdTest() {
        SongDTO songDTO = new SongDTO();
        songDTO.setId(1L);
        songDTO.setAlbumId(1L);

        Song song = new Song();
        song.setAlbum(new Album());

        when(albumRepository.findById(1L)).thenReturn(Optional.of(new Album()));
        when(songMapper.songDTOToSong(songDTO)).thenReturn(song);
        when(songRepository.save(song)).thenReturn(song);
        when(songMapper.songToSongDTO(song)).thenReturn(songDTO);

        SongDTO updatedSong = songService.updateSongById(1L, songDTO);

        assertNotNull(updatedSong);
        assertEquals(1L, updatedSong.getId());
        assertEquals(1L, updatedSong.getAlbumId());
    }

    @Test
    @DisplayName("Remove Song By Id Test")
    void removeSongByIdTest() {
        Song song = new Song();
        song.setId(1L);

        when(songRepository.findById(1L)).thenReturn(Optional.of(song));

        doNothing().when(songRepository).delete(song);
        assertDoesNotThrow(() -> songService.removeSongById(1L));
    }
}